@@ -1007,10 +1007,12 @@ check_external_diff(diffio_T *diffio)
 
 		    for (;;)
 		    {
-			// There must be a line that contains "1c1".
+			// For normal diff there must be a line that contains
+			// "1c1".  For unified diff "@@ -1 +1 @@".
 			if (vim_fgets(linebuf, LBUFLEN, fd))
 			    break;
-			if (STRNCMP(linebuf, "1c1", 3) == 0)
+			if (STRNCMP(linebuf, "1c1", 3) == 0
+				|| STRNCMP(linebuf, "@@ -1 +1 @@", 11) == 0)
 			    ok = TRUE;
 		    }
 		    fclose(fd);
