@@ -207,7 +207,7 @@ im_preedit_window_set_position(void)
     if (preedit_window == NULL)
 	return;
 
-    gui_gtk_get_screen_geom_of_win(gui.drawarea,
+    gui_gtk_get_screen_geom_of_win(gui.drawarea, 0, 0,
 			  &screen_x, &screen_y, &screen_width, &screen_height);
     gdk_window_get_origin(gtk_widget_get_window(gui.drawarea), &x, &y);
     gtk_window_get_size(GTK_WINDOW(preedit_window), &width, &height);
